use strict;
use warnings;

use Module::Build;

my $build = Module::Build->new(
	module_name => 'Db::Access',
	dist_author => 'Eugen Konkov',
	requires => {
		'DDP'                    =>  0,
		'DBD::Pg'                =>  0,
		'DBIx::Class'            =>  0,
		'DBIx::Class::Migration' =>  0,
		'DBIx::Class::Report'    =>  0,
	},
	recommends         => {
		'Mojolicious' =>  0,
	},
	test_requires      => {
		'Test2'        =>  0,
		'App::Prove'   =>  0,
		'Import::Into' =>  0,
	},
	configure_requires => {
		'Module::Build' =>  0,
	},
	license            => 'MIT',
	meta_merge         => {
		resources => {
			repository =>  'https://github.com/KES777/Db-Access',
			bugtracker =>  'https://github.com/KES777/Db-Access/issues',
		},
	},
);

$build->create_build_script;
